<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">

	<xsl:import href="../../custom/corporate_defaults.xsl"/>
	<xsl:import href="../../base/report/rml_template.xsl"/>

	<xsl:template match="/">
		<xsl:call-template name="rml"/>
	</xsl:template>

	<xsl:template name="stylesheet">
		<paraStyle name="name" fontName="Helvetica-Bold" fontSize="16"/>
		
		<blockTableStyle id="products">
			 <blockFont name="Helvetica-BoldOblique" size="12" start="0,0" stop="2,0"/>
			 <blockValign value="TOP"/>
			 <blockAlignment value="RIGHT" start="-1,0" stop="-1,-1"/>
			 <lineStyle kind="LINEBELOW" start="0,0" stop="-1,0"/>
		</blockTableStyle>
		
		<blockTableStyle id="product-totals">
			 <blockValign value="TOP"/>
			 <blockAlignment value="RIGHT"/>
			 <lineStyle kind="LINEABOVE" start="-1,0" stop="-1,0"/>
			 <lineStyle kind="LINEABOVE" start="-1,-1" stop="-1,-1"/>
		</blockTableStyle>
	</xsl:template>

	<xsl:template name="story">
		<xsl:apply-templates select="achat-bordereau-list"/>
	</xsl:template>

	<xsl:template match="achat-bordereau-list">
		<xsl:apply-templates select="achat-bordereau">
			<xsl:sort order="ascending" select="client_info/name"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="client_info">
		<para>
			<b>
				<xsl:value-of select="title" />
				<xsl:text> </xsl:text>
				<xsl:value-of select="name"/>
			</b>
		</para>
		<para><xsl:value-of select="street"/></para>
		<para><xsl:value-of select="street2"/></para>
		<para>
			<xsl:value-of select="zip"/><xsl:text> </xsl:text>
			<xsl:value-of select="city"/>
		</para>
		<para><xsl:value-of select="country"/></para>
	</xsl:template>

	<xsl:template match="achat-bordereau">
		<xsl:apply-templates select="client_info"/>
		
		<setNextTemplate name="other_pages"/>
		<nextFrame/>
		
		<para style="name"><xsl:value-of select="firstname"/></para>

		<spacer height="1cm" width="1cm" length="1cm"/>
		
		<para>
			<b t="1">Document</b>: <xsl:value-of select="subject"/>
		</para><para>
			<b t="1">Auction</b>: <xsl:value-of select="title"/>
		</para>
		
		<xsl:if test="client_info">
			<para>
				<b t="1">Contact</b>: 
				<xsl:value-of select="client_info/phone"/> 
				<xsl:if test="number(string-length(client_info/mobile) &gt; 0) + number(string-length(client_info/phone) &gt; 0) = 2">
					<xsl:text> - </xsl:text>
				</xsl:if>
				<xsl:value-of select="client_info/mobile"/>
			</para><para>
				<b t="1">Reference</b>: <xsl:value-of select="client_info/ref"/>
			</para>
		</xsl:if>
		
		<spacer height="1cm" width="1cm" length="1cm"/>
		
		<xsl:apply-templates select="achat-products"/>

		<setNextTemplate name="first_page"/>
		<pageBreak/>
	</xsl:template>

	<xsl:template match="achat-products">
		<blockTable colWidths="3.1cm,1.2cm,8.6cm,1.5cm,2.2cm" style="products">
			<tr>
				<td/>
				<td t="1">Ref</td><td t="1">Descriptie</td><td t="1">Bet.?</td><td t="1">Adj.(EUR)</td>
			</tr>
			<xsl:apply-templates select="product"/>
		</blockTable>
		<condPageBreak height="3.2cm"/>
		<blockTable colWidths="3.1,1.2cm,8.6cm,1.5cm,2.2cm" style="product-totals">
			<tr>
				<td/>
				<td/>
				<td/>
				<td t="1">Subtotal:</td>
				<td><xsl:value-of select="format-number(sum(product[price != '']/price), '#,##0.00')"/></td>
			</tr>
			<xsl:apply-templates select="cost"/>
			<tr>
				<td/>
				<td/>
				<td/>
				<td t="1">Total:</td>
				<td><xsl:value-of select="format-number(sum(product[price != '']/price) + sum(cost/amount), '#,##0.00')"/></td>
			</tr>
		</blockTable>
	</xsl:template>

	<xsl:template match="cost">
		<tr>
			<td/>
			<td/>
			<td/>
			<td><xsl:value-of select="name"/>:</td>
			<td><xsl:value-of select="format-number(amount, '#,##0.00')"/></td>
		</tr>
	</xsl:template>
	
	<xsl:template match="product">
		<tr>
			<td>
				<xsl:if test="image">
					<image width="2.5cm" height="2.2cm">
						<xsl:attribute name="name"><xsl:value-of select="image"/></xsl:attribute>
					</image>
				</xsl:if>
			</td>
<td><xsl:value-of select="ref"/></td>
			<td>
				<para>
					<b><xsl:value-of select="title"/><xsl:text>. </xsl:text></b>
					<xsl:value-of select="name"/>
				</para>
			</td>
			<td><xsl:if test="state!='draft'"><xsl:text>X</xsl:text></xsl:if></td>
			<td>
				<xsl:if test="price!=''">
					<xsl:value-of select="format-number(price, '#,##0.00')"/>
				</xsl:if>
			</td>
		</tr>
	</xsl:template>

</xsl:stylesheet>
