##############################################################################
#
# Copyright (c) 2005 TINY SPRL. (http://tiny.be) All Rights Reserved.
#
# $Id: mrp.py 1254 2005-08-30 14:39:11Z nicoe $
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields
from osv import osv
import ir

import netsvc
import time
from mx import DateTime

#----------------------------------------------------------
# Workcenters
#----------------------------------------------------------
# capacity_hour : capacity per hour. default: 1.0. 
#          Eg: If 5 concurrent operations at one time: capacity = 5 (because 5 employees)
# unit_per_cycle : how many units are produced for one cycle
#
# TODO: Work Center may be recursive ?
#
class mrp_workcenter(osv.osv):
	_name = 'mrp.workcenter'
	_description = 'Workcenter'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'active': fields.boolean('Active'),
		'type': fields.selection([('machine','Machine'),('hr','Human Ressource'),('tool','Tool')], 'Type', required=True),
		'code': fields.char('Code', size=8),

		'timesheet_id': fields.many2one('hr.timesheet', 'Timesheet'),
		'note': fields.text('Description'),

		'capacity_per_cycle': fields.float('Capacity per Cycle'),

		'time_cycle': fields.float('Time for 1 cycle (hour)'),
		'time_start': fields.float('Time before prod.'),
		'time_stop': fields.float('Time after prod.'),
		'time_efficiency': fields.float('Time Efficiency'),

		'costs_hour': fields.float('Cost per hour'),
		'costs_hour_account_id': fields.many2one('account.account', 'Hour Account', required=True, domain=[('type','=','expense')]),
		'costs_cycle': fields.float('Cost per cycle'),
		'costs_cycle_account_id': fields.many2one('account.account', 'Cycle Account', required=True, domain=[('type','=','expense')]),
		'project_id': fields.many2one('account.project', 'Profit/Cost center')
	}

	_defaults = {
		'active': lambda x,y,z: 1,
		'type': lambda x,y,z: 'machine',
		'time_efficiency': lambda x,y,z: 1,
		'capacity_per_cycle': lambda x,y,z: 1,
	}
mrp_workcenter()


class mrp_property_group(osv.osv):
	_name = 'mrp.property.group'
	_description = 'Property Group'
	_columns = {
		'name': fields.char('Property Group', size=64, required=True),
		'description': fields.text('Description'),
	}
mrp_property_group()

class mrp_property(osv.osv):
	_name = 'mrp.property'
	_description = 'Property'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'composition': fields.selection([('min','min'),('max','max'),('plus','plus')], 'Properties composition', required=True),
		'group_id': fields.many2one('mrp.property.group', 'Property Group', required=True),
		'description': fields.text('Description'),
	}
	_defaults = {
		'composition': lambda x,y,z: 'min',
	}
mrp_property()

class mrp_routing(osv.osv):
	_name = 'mrp.routing'
	_description = 'Routing'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'active': fields.boolean('Active'),
		'code': fields.char('Code', size=8),

		'note': fields.text('Description'),
		'workcenter_lines': fields.one2many('mrp.routing.workcenter', 'routing_id', 'Workcenters'),

		'location_id': fields.many2one('stock.location', 'Production Location'),
	}
	_defaults = {
		'active': lambda x,y,z: 1,
	}
mrp_routing()

class mrp_routing_workcenter(osv.osv):
	_name = 'mrp.routing.workcenter'
	_description = 'Routing workcenter usage'
	_columns = {
		'workcenter_id': fields.many2one('mrp.workcenter', 'Workcenter', required=True),
		'name': fields.char('Name', size=64, required=True),
		'cycle_nbr': fields.float('Number of cycle', required=True),
		'routing_id': fields.many2one('mrp.routing', 'Parent Routing'),
	}
	_defaults = {
		'cycle_nbr': lambda x,y,z: 1.0,
	}
mrp_routing_workcenter()

class mrp_bom(osv.osv):
	_name = 'mrp.bom'
	_description = 'Bill of Material'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'code': fields.char('Code', size=16),
		'active': fields.boolean('Active'),
		'type': fields.selection([('normal','Normal BOM'),('phantom','Phantom')],'BOM Type',required=True),

		'date_start': fields.date('Valid from'),
		'date_stop': fields.date('Valid until'),

		'sequence': fields.integer('Sequence'),

		'position': fields.char('Position #', size=64),
		'product_id': fields.many2one('product.product', 'Product', required=True),
		'product_qty': fields.float('Product Qty', required=True),
		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),

		'product_rounding': fields.float('Product Rounding'),

		'product_efficiency': fields.float('Product Efficiency', required=True),

		'bom_lines': fields.one2many('mrp.bom', 'bom_id', 'BOM Lines'),
		'bom_id': fields.many2one('mrp.bom', 'Parent BOM'),

		'routing_id': fields.many2one('mrp.routing', 'Routing'),

		'property_ids': fields.many2many('mrp.property', 'mrp_bom_property_rel', 'bom_id','property_id', 'Properties'),

		'revision_ids': fields.one2many('mrp.bom.revision', 'bom_id', 'BOM Revisions'),
	}
	_defaults = {
		'active': lambda x,y,z: 1,
		'product_efficiency': lambda x,y,z: 1,
		'product_rounding': lambda x,y,z: 1,
		'type': lambda x,y,z: 'normal',
	}
	_order = "name"
	def onchange_product_id(self, cr, uid, ids, product_id,name):
		if product_id:
			prod=self.pool.get('product.product').browse(cr,uid,[product_id])[0]
			v = {'product_uom':prod.product_uom.id}
			if not name:
				v['name'] = prod.name
			return {'value': v}
		return {}

	def _bom_find(self, cr, uid, product_id, product_uom, properties = []):
		bom_result = False
		result = self.pool.get('mrp.bom').search(cr, uid, [('product_id','=',product_id), ('bom_id','=',False)])
		if len(result):
			#
			# TODO: choisir en fonction des properties
			#
			return result[0]
		return False

	def _bom_explode(self, cr, uid, bom, factor, properties, addthis=False, level=10):
		factor = factor * (bom.product_efficiency or 1.0)
		factor = rounding(factor, bom.product_rounding)
		result = []
		result2 = []
		if bom.type=='phantom' and not bom.bom_lines:
			newbom = self._bom_find(cr, uid, bom.product_id.id, bom.product_uom.id, properties)
			if newbom:
				res = self._bom_explode(cr, uid, self.browse(cr, uid, [newbom])[0], factor*bom.product_qty, properties, addthis=True, level=level+10)
				result = result + res[0]
				result2 = result2 + res[1]
			else:
				return [],[]
		else:
			if addthis and not bom.bom_lines:
				result.append(
				{
					'name': bom.product_id.name,
					'product_id': bom.product_id.id,
					'product_qty': bom.product_qty * factor,
					'product_uom': bom.product_uom.id,
				})
			if bom.routing_id:
				for wc_use in bom.routing_id.workcenter_lines:
					wc = wc_use.workcenter_id
					cycle = factor * wc_use.cycle_nbr
					result2.append({
						'name': bom.routing_id.name,
						'workcenter_id': wc.id,
						'sequence': level,
						'cycle': cycle,
						'hour': (wc.time_start+wc.time_stop+cycle*wc.time_cycle) * (wc.time_efficiency or 1.0),
					})
			for bom2 in bom.bom_lines:
				res = self._bom_explode(cr, uid, bom2, factor, properties, addthis=True, level=level+10)
				result = result + res[0]
				result2 = result2 + res[1]
		return result, result2

	def copy(self, cr, uid, ids):
		ret = []
		boms = self.browse(cr, uid, ids)
		for bom in boms:
			nid = super(mrp_bom, self).copy(cr, uid, bom.id)
			self.write(cr, uid, [nid], {'name' : '%s (copy)' % bom.name})
			ret.append(nid)
			childs = [sbom.id for sbom in bom.bom_lines]
			nchilds = self.copy(cr, uid, childs)
			self.write(cr, uid, nchilds, {'bom_id' : nid})
		return ret

mrp_bom()

class mrp_bom_revision(osv.osv):
	_name = 'mrp.bom.revision'
	_description = 'Bill of material revisions'
	_columns = {
		'name' : fields.char('Modification', size=64, required=True),
		'indice' : fields.char('Indice', size=16, required=True),
		'bom_id' : fields.many2one('mrp.bom', 'BOM'),
		}
mrp_bom_revision()

def rounding(f, r):
	if not r:
		return f
	return round(f / r) * r

class mrp_production(osv.osv):
	_name = 'mrp.production'
	_description = 'Production'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'origin': fields.char('Origin', size=64),
		'priority': fields.selection([('0','Not urgent'),('1','Normal'),('2','Urgent'),('3','Very Urgent')], 'Priority'),

		'product_id': fields.many2one('product.product', 'Product', required=True),
		'product_qty': fields.float('Product Qty', required=True),
		'product_uom': fields.many2one('product.uom', 'Product UOM'),

		'location_src_id': fields.many2one('stock.location', 'Source Location', required=True),
		'location_dest_id': fields.many2one('stock.location', 'Destination Location', required=True),

		'date_planned': fields.date('Planned date', required=True),
		'date_start': fields.datetime('Start Date'),
		'date_finnished': fields.datetime('End Date'),

		'bom_id': fields.many2one('mrp.bom', 'Bill of Material', domain=[('bom_id','=','False')]),

		'picking_id': fields.many2one('stock.picking', 'Picking list'),
		'reservation_id': fields.many2one('stock.reservation', 'Reservation'),
		'reservation_prod_id': fields.many2one('stock.reservation', 'Reservation'),

		'lot_created_id': fields.many2one('stock.lot.line', 'Lot Created'),

		'lot_lines': fields.one2many('mrp.production.lot.line', 'production_id', 'Lots Consummed'),
		'reservation_lines': fields.many2many('stock.reservation', 'mrp_production_reservation_rel', 'production_id', 'reservation_id', 'Lots reserved'),

		'product_lines': fields.one2many('mrp.production.product.line', 'production_id', 'Material Planned'),
		'workcenter_lines': fields.one2many('mrp.production.workcenter.line', 'production_id', 'Workcenters Utilisation'),

		'state': fields.selection([('draft','Draft'),('picking_except', 'Picking Exception'),('confirmed','Waiting Goods'),('ready','Ready to Produce'),('in_production','In Production'),('cancel','Canceled'),('done','Done')],'State', readonly=True)
	}
	_defaults = {
		'active': lambda x,y,z: 1,
		'priority': lambda x,y,z: '1',
		'type': lambda x,y,z: 'make_to_stock',
		'state': lambda x,y,z: 'draft',
	}
	_order = 'date_planned desc, priority desc';
	def product_id_change(self, cr, uid, ids, product):
		if not product:
			return {}
		res = self.pool.get('product.product').read(cr, uid, [product], ['uom_id'])[0]
		uom = res['uom_id'] and res['uom_id'][0]
		result = {'product_uom':uom}
		return {'value':result}

	def action_picking_except(self, cr, uid, ids):
		self.write(cr, uid, ids, {'state':'picking_except'})
		return True

	def action_compute(self, cr, uid, ids, properties=[]):
		ok = True
		for production in self.browse(cr, uid, ids):
			cr.execute('delete from mrp_production_product_line where production_id=%d', (production.id,))
			cr.execute('delete from mrp_production_workcenter_line where production_id=%d', (production.id,))
			bom_point = production.bom_id
			bom_id = production.bom_id.id
			if not bom_point:
				bom_id = self.pool.get('mrp.bom')._bom_find(cr, uid, production.product_id.id, production.product_uom.id, properties)
				bom_point = self.pool.get('mrp.bom').browse(cr, uid, [bom_id])[0]

			if bom_point.routing_id and bom_point.routing_id.location_id:
				self.write(cr, uid, [production.id], {'location_src_id': bom_point.routing_id.location_id.id})

			factor = production.product_qty
			res = self.pool.get('mrp.bom')._bom_explode(cr, uid, bom_point, factor / bom_point.product_qty, properties)
			results = res[0]
			results2 = res[1]
			if not results:
				ok=False
			for line in results:
				line['production_id'] = production.id
				print 'CREATING', line
				self.pool.get('mrp.production.product.line').create(cr, uid, line)
			for line in results2:
				line['production_id'] = production.id
				self.pool.get('mrp.production.workcenter.line').create(cr, uid, line)
		return bom_id
		
	def action_cancel(self, cr, uid, ids):
		for production in self.browse(cr, uid, ids):
			self.pool.get('stock.reservation').action_cancel(cr, uid, [production.reservation_prod_id.id])
			self.pool.get('mrp.production.lot.line').unlink(cr, uid, [x.id for x in production.lot_lines])
			self.pool.get('stock.reservation').action_cancel(cr, uid, [x.id for x in production.reservation_lines])
		self.write(cr, uid, ids, {'state':'cancel','reservation_lines':[]})
		return True

	#
	# BUGFIX: may be a bug here; lot_lines are unreserved for a few seconds;
	#         between the end of the picking list and the call to this function
	#
	def action_ready(self, cr, uid, ids):
		self.write(cr, uid, ids, {'state':'ready'})
		for production in self.browse(cr, uid, ids):
			for reserv in production.reservation_lines:
				for lot in reserv.lot_line_ids:
					self.pool.get('mrp.production.lot.line').create(cr, uid, {
						'name': lot.name,
						'tracking': lot.tracking,
						'serial': lot.serial,
						'product_id': lot.product_id.id,
						'product_qty': lot.product_qty,
						'product_uom': lot.product_uom.id,
						'production_id': production.id,
						'lot_id': lot.id
					})
		return True

	#
	# Review materials in function in_prod and prod_end.
	#
	def action_production_end(self, cr, uid, ids):
		reserv_ids = []
		for production in self.browse(cr, uid, ids):
			price = 0.0
			for p in production.lot_lines:
				price += p.lot_id.product_price * p.product_qty * p.product_uom.factor
			for p in production.workcenter_lines:
				price += (p.cycle * p.workcenter_id.costs_cycle or 0.0) + (p.hour * p.workcenter_id.costs_hour or 0.0)
			price = price / (production.product_qty * production.product_uom.factor)

			id = self.pool.get('stock.lot.line').create(cr, uid, {
				'name': 'PROD:'+production.name,
				'product_id': production.product_id.id,
				'product_uom': production.product_uom.id,
				'product_qty': production.product_qty,
				'product_price': price or 0.0,
				'location_id': production.location_dest_id.id,
				'reservation_id': production.reservation_id.id
			})

			self.write(cr, uid, [production.id], {'lot_created_id':id})
			ids2 = []
			for res in production.reservation_lines:
				ids2.extend([x.id for x in res.lot_line_ids])
			for i in ids2:
				cr.execute('insert into stock_lot_line_rel (parent_id,child_id) values (%d,%d)', (i, id))

			self.pool.get('stock.reservation').write(cr, uid, [production.reservation_id.id], {'state':'confirmed'})
			self.pool.get('stock.reservation').check_assign(cr, uid, [production.reservation_id.id])
			for lot in production.lot_lines:
				reserv_ids.append( lot.lot_id.reservation_id.id)
			self.pool.get('stock.reservation').write(cr, uid, [production.reservation_prod_id.id], {'state':'done'})
		self.pool.get('stock.reservation').action_done(cr, uid, filter(lambda x:x, reserv_ids))
		self.write(cr,  uid, ids, {'state':'done'})
		return True

	def action_in_production(self, cr, uid, ids):
		for production in self.browse(cr, uid, ids):
			ids2 = []
			for res in production.reservation_lines:
				ids2.extend([x.id for x in res.lot_line_ids])
			self.pool.get('stock.lot.line').write(cr, uid, ids2, {'location_id':False})
		self.write(cr, uid, ids, {'state':'in_production'})
		return True

	def action_confirm(self, cr, uid, ids):
		picking_id=False
		for production in self.browse(cr, uid, ids):
			if not production.product_lines:
				self.action_compute(cr, uid, [production.id])
				production = self.browse(cr, uid, [production.id])[0]
			picking_id = self.pool.get('stock.picking').create(cr, uid, {
				'name':'PICK:'+production.name,
				'origin':'PROD:'+str(production.id)+':'+production.name,
				'type':'internal',
				'move_type': 'one',
				'state': 'auto',
				'auto_picking': True,
				'loc_dest_id': production.location_src_id,
			})
			toconfirm = True
			
			delay = {}
			for line in production.workcenter_lines:
				delay[line.sequence] = max(line.hour, delay.get(line.sequence,0))
			totaldelay = reduce(lambda x,y: x+y, delay.values(), 0)
			res_final_id = self.pool.get('stock.reservation').create(cr, uid, {
				'name':'PROD:'+production.name,
				'date_planned': production.date_planned,
				'product_id': production.product_id.id,
				'product_qty': production.product_qty,
				'product_uom': production.product_uom.id,
				'location_id': False,
				'location_dest_id': production.location_dest_id.id,
				'state': 'waiting',
			})
			self.write(cr, uid, [production.id], {'reservation_prod_id':res_final_id})
			reservations = []
			for line in production.product_lines:
				print 'CREATE ONE LINE'
				res_dest_id = self.pool.get('stock.reservation').create(cr, uid, {
					'name':'PROD:'+production.name,
					'date_planned': production.date_planned,
					'product_id': line.product_id.id,
					'product_qty': line.product_qty,
					'product_uom': line.product_uom.id,
					'location_id': production.location_src_id.id,
					'location_dest_id': False,
					'reservation_dest_id': res_final_id,
					'state': 'waiting',
				})
				reservations.append(res_dest_id)
				reservation_id = self.pool.get('stock.reservation').create(cr, uid, {
					'name':'PROD:'+production.name,
					'picking_id':picking_id,
					'product_id': line.product_id.id,
					'product_qty': line.product_qty,
					'product_uom': line.product_uom.id,
					'date_planned': production.date_planned, # TODO: - production delay
					'reservation_dest_id': res_dest_id,
					'location_id': production.location_src_id.id,
					'location_dest_id': production.location_src_id.id,
					'state': 'waiting',
				})
				newdate = DateTime.strptime(production.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=totaldelay/3)
				proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
					'name': 'PROC:'+production.name,
					'origin': 'PROD:'+str(production.id)+':'+production.name,
					#TODO: A deduire: le delai de production
					'date_planned': newdate.strftime('%Y-%m-%d'),
					'product_id': line.product_id.id,
					'product_qty': line.product_qty,
					'product_uom': line.product_uom.id,
					'location_id': production.location_src_id.id,
					'procure_method': line.product_id.procure_method,
					'reservation_id': reservation_id, 
				})
				wf_service = netsvc.LocalService("workflow")
				wf_service.trg_validate(uid, 'mrp.procurement', proc_id, 'button_confirm', cr)
			if toconfirm:
				wf_service = netsvc.LocalService("workflow")
				wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
			self.write(cr, uid, [production.id], {'picking_id':picking_id, 'reservation_lines': reservations, 'state':'confirmed'})
		return picking_id
mrp_production()

class mrp_production_lot_line(osv.osv):
	_name = 'mrp.production.lot.line'
	_description = 'Production Lots Used'
	_columns = {
		'name': fields.char('Name', size=64, readonly=True),
		'tracking': fields.char('Tracking', size=64, readonly=True),
		'serial': fields.char('Serial', size=64, readonly=True),
		'product_id': fields.many2one('product.product', 'Product', readonly=True),
		'product_qty': fields.float('Product Qty', readonly=True),
		'product_uom': fields.many2one('product.uom', 'Product UOM', readonly=True),
		'production_id': fields.many2one('mrp.production', 'Production Order', required=True),
		'lot_id': fields.many2one('stock.lot.line', 'Lot', required=True),
	}
	def onchange_lot_line_id(self, cr, uid, arg, lot_line_id):
		if not lot_line_id:
			return {'value':{'product_id': False, 'product_uom':False, 'product_qty': False, 'tracking': False, 'serial':False}}
		res = self.pool.get('stock.lot.line').browse(cr, uid, [lot_line_id])[0]
		return {'value':{'product_id': res.product_id.id, 'product_uom':res.product_uom.id, 'product_qty': res.product_qty, 'tracking': res.tracking, 'serial':res.serial}}
mrp_production_lot_line()

class mrp_production_workcenter_line(osv.osv):
	_name = 'mrp.production.workcenter.line'
	_description = 'Production workcenters used'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'workcenter_id': fields.many2one('mrp.workcenter', 'Workcenter', required=True),
		'cycle': fields.float('Nbr of cycle'),
		'hour': fields.float('Nbr of hour'),
		'sequence': fields.integer('Sequence', required=True),
		'production_id': fields.many2one('mrp.production', 'Production Order'),
	}
	_defaults = {
		'sequence': lambda x,y,z: 1,
		'hour': lambda x,y,z: 0,
		'cycle': lambda x,y,z: 0,
	}
mrp_production_workcenter_line()

class mrp_production_product_line(osv.osv):
	_name = 'mrp.production.product.line'
	_description = 'Production planned products'
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'product_id': fields.many2one('product.product', 'Product', required=True),
		'product_qty': fields.float('Product Qty', required=True),
		'product_uom': fields.many2one('product.uom', 'Product UOM'),
		'production_id': fields.many2one('mrp.production', 'Production Order'),
	}
mrp_production_product_line()

# ------------------------------------------------------------------
# Procurement
# ------------------------------------------------------------------
#
# Produce, Buy or Find products and place a reservation
#     then wizard for picking lists & move
#
class mrp_procurement(osv.osv):
	_name = "mrp.procurement"
	_description = "Procurement"
	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'origin': fields.char('Origin', size=64),
		'priority': fields.selection([('0','Not urgent'),('1','Normal'),('2','Urgent'),('3','Very Urgent')], 'Priority', required=True),
		'date_planned': fields.date('Date Promised', required=True),
		'date_close': fields.date('Date Closed'),
		'product_id': fields.many2one('product.product', 'Product', required=True ),
		'product_qty': fields.float('Quantity', required=True),
		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
		'reservation_id': fields.many2one('stock.reservation', 'Reservation', ondelete='cascade'),
		'location_id': fields.many2one('stock.location', 'Location', required=True),
		'procure_method': fields.selection([('make_to_stock','from stock'),('make_to_order','on order')], 'Procurement Method', states={'draft':[('readonly',False)]}, readonly=True),

		'purchase_id': fields.many2one('purchase.order', 'Purchase Order'),
		'purchase_line_id': fields.many2one('purchase.order.line', 'Purchase Order Line'),

		'property_ids': fields.many2many('mrp.property', 'mrp_procurement_property_rel', 'procurement_id','property_id', 'Properties'),

		'message': fields.char('Latest error', size=64),
		'state': fields.selection([('draft','Draft'),('confirmed','Confirmed'),('exception','Exception'),('running','Running'),('cancel','Cancel'),('done','Done')], 'State')
	}
	_defaults = {
		'state': lambda x,y,z: 'draft',
		'priority': lambda x,y,z: '1',
		'procure_method': lambda x,y,z: 'make_to_order',
	}
	def check_produce(self, cr, uid, ids):
		for procurement in self.browse(cr, uid, ids):
			if procurement.product_id.product_tmpl_id.supply_method=='buy':
				return False
			#if not procurement.product_id.product_tmpl_id.seller_id:
			#	self.write(cr, uid, [procurement.id], {'message':'Not BOM defined for this product !'})
			#	return False
		return True

	def check_buy(self, cr, uid, ids):
		for procurement in self.browse(cr, uid, ids):
			if procurement.product_id.product_tmpl_id.supply_method=='produce':
				return False
			if not procurement.product_id.product_tmpl_id.seller_id:
				self.write(cr, uid, [procurement.id], {'message':'Not default seller defined for this product !'})
				return False
		return True

	def test_cancel(self, cr, uid, ids):
		for record in self.browse(cr, uid, ids):
			if record.reservation_id.state=='cancel':
				return True
		return False

	def action_confirm(self, cr, uid, ids):
		for procurement in self.browse(cr, uid, ids):
			if not procurement.reservation_id:
				id = self.pool.get('stock.reservation').create(cr, uid, {
					'name': 'PROC:'+procurement.name,
					'location_id': procurement.location_id.id,
					'location_dest_id': procurement.location_id.id,
					'product_id': procurement.product_id.id,
					'product_qty':procurement.product_qty,
					'state':'confirmed',
					'product_uom': procurement.product_uom.id,
					'date_planned': procurement.date_planned
				})
				self.write(cr, uid, [procurement.id], {'reservation_id': id})
			else:
				if procurement.procure_method=='make_to_stock' and procurement.reservation_id.state in ('waiting',):
					id = self.pool.get('stock.reservation').write(cr, uid, [procurement.reservation_id.id], {'state':'confirmed'})
		self.write(cr, uid, ids, {'state':'confirmed','message':''})
		return True

	def action_reserve_create(self, cr, uid, ids):
		return True
		self.write(cr, uid, ids, {'state':'confirmed'})
		for procurement in self.browse(cr, uid, ids):
			if not procurement.reservation_id:
				id = self.pool.get('stock.reservation').create(cr, uid, {
					'name': 'PROC:'+procurement.name,
					'location_id': procurement.location_id.id,
					'location_dest_id': procurement.location_id.id,
					'product_id': procurement.product_id.id,
					'product_qty':procurement.product_qty,
					'state':'confirmed',
					'product_uom': procurement.product_uom.id,
					'date_planned': procurement.date_planned})
				self.write(cr, uid, [procurement.id], {'reservation_id': id})
			else:
				id = self.pool.get('stock.reservation').write(cr, uid, [procurement.reservation_id.id], {'state':'confirmed'})
		return True

	def action_reserve_assign(self, cr, uid, ids):
		ok = True
		for procurement in self.browse(cr, uid, ids):
			id = procurement.reservation_id.id
			if not (procurement.reservation_id.state in ('done','assigned','cancel')):
				ok = ok and self.pool.get('stock.reservation').action_assign(cr, uid, [id])
#				if ok:
#					self.write(cr, uid, [procurement.id], {'state':'running'})
		return ok

	def action_produce_assign(self, cr, uid, ids):
		produce_id = False
		for procurement in self.browse(cr, uid, ids):
			res_id = procurement.reservation_id
			loc_id = procurement.location_id.id
			produce_id = self.pool.get('mrp.production').create(cr, uid, {
				'name': procurement.name,
				'origin': 'PROC:'+str(procurement.id)+':'+procurement.name,
				'product_id': procurement.product_id.id,
				'product_qty': procurement.product_qty,
				'product_uom': procurement.product_uom.id,
				'location_src_id': procurement.location_id.id,
				'location_dest_id': procurement.location_id.id,
				'date_planned': procurement.date_planned, 
				'reservation_id': res_id,
			})
			self.write(cr, uid, [procurement.id], {'state':'running'})
			bom_result = self.pool.get('mrp.production').action_compute(cr, uid, [produce_id], properties=procurement.property_ids)
			if bom_result:
				wf_service = netsvc.LocalService("workflow")
				wf_service.trg_validate(uid, 'mrp.production', produce_id, 'button_confirm', cr)
		return produce_id

	def action_po_assign(self, cr, uid, ids):
		purchase_id = False
		for procurement in self.browse(cr, uid, ids):
			res_id = procurement.reservation_id
			#res_id = self.pool.get('stock.reservation').create(cr, uid, {'name': 'PROCURE:'+procurement.name, 'location_id': procurement.location_id.id, 'product_id': procurement.product_id.id, 'product_qty':procurement.product_qty,'state':'waiting'})

			partner = procurement.product_id.product_tmpl_id.seller_id
			partner_id = partner.id
			address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
			pricelist_id=ir.ir_get(cr,uid,'meta','product.pricelist.purchase', [('res.partner',partner_id)])[0][2]
			price = self.pool.get('product.pricelist').price_get(cr,uid,[pricelist_id], procurement.product_id.id, procurement.product_qty, 'standard')[pricelist_id]
			line = {
				'name': procurement.name,
				'product_qty': procurement.product_qty,
				'product_id': procurement.product_id.id,
				'product_uom': procurement.product_uom.id,
				'price_unit': price,
				'date_planned': procurement.date_planned,
				'taxes_id': [x.id for x in procurement.product_id.product_tmpl_id.taxes_id],
				'reservation_dest_id': res_id,
			}

			# TODO: may use a draft order instead
			purchase_id = self.pool.get('purchase.order').create(cr, uid, {
				'origin': 'PROC:'+str(procurement.id)+':'+str(procurement.name),
				'partner_id':partner_id,
				'partner_address_id':address_id,
				'location_id': procurement.location_id.id,
				'pricelist_id': pricelist_id,
				'order_line': [(0,0,line)]
			})
			self.write(cr, uid, [procurement.id], {'state':'running','purchase_id':purchase_id})
		return purchase_id

	def action_cancel(self, cr, uid, ids):
		todo = []
		for proc in self.browse(cr, uid, ids):
			if proc.reservation_id:
				todo.append(proc.reservation_id.id)
		if len(todo):
			self.pool.get('stock.reservation').action_cancel(cr, uid, ids)
		self.write(cr, uid, ids, {'state':'cancel'})
		return True

	def action_check_finnished(self, cr, uid, ids):
		return True

	def action_check(self, cr, uid, ids):
		ok = False
		for procurement in self.browse(cr, uid, ids):
			if procurement.reservation_id.state=='assigned' or procurement.reservation_id.state=='done':
				self.action_done(cr, uid, [procurement.id])
				ok = True
		return ok

	def action_done(self, cr, uid, ids):
		return self.write(cr, uid, ids, {'state':'done', 'date_close':time.strftime('%Y-%m-%d')})
mrp_procurement()

class stock_warehouse_orderpoint(osv.osv):
	_name = "stock.warehouse.orderpoint"
	_description = "Warehouse Orderpoint"
	_columns = {
		'name': fields.char('Name', size=32, required=True),
		'active': fields.boolean('Active'),
		'logic': fields.selection([('max','Order to Max'),('price','Best price (not yet active!)')], 'Reordering Mode', required=True),
		'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', required=True ),
		'product_id': fields.many2one('product.product', 'Product', required=True ),
		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True ),
		'product_min_qty': fields.float('Min Quantity', required=True),
		'product_max_qty': fields.float('Max Quantity', required=True),
		'product_max_qty': fields.float('Max Quantity', required=True),
		'qty_multiple': fields.integer('Qty Multiple', required=True),
		'procurement_id': fields.many2one('mrp.procurement', 'Purchase Order')
	}
	_defaults = {
		'active': lambda x,y,z: 1,
		'logic': lambda x,y,z: 'max',
		'qty_multiple': lambda x,y,z: 1
	}
stock_warehouse_orderpoint()
